/*
TNKernel LP real-time kernel - examples

Copyright  2004,2008 Yuri Tiomkin
All rights reserved.

Permission to use, copy, modify, and distribute this software in source
and binary forms and its documentation for any purpose and without fee
is hereby granted, provided that the above copyright notice appear
in all copies and that both that copyright notice and this permission
notice appear in supporting documentation.

THIS SOFTWARE IS PROVIDED BY THE YURI TIOMKIN AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL YURI TIOMKIN OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

/*
    tect2 project (CPU - MC9S08GT60A)

    Processing  buttons & tilted motion sensor. Using Timer and Kbd Interrupt
  to wake-up from the "deep sleeping"

    Interrupt vectors are remapped to the RAM (for the sharing with bootloader;
  bootloader is used for the field firmware upgrading)

*/


#include "../MC9S08GT60A.h"
#include "utils.h"
#include "../../tnkernel_lp/tn.h"
#include "int_vectors.h"

//-- Specific Freescale CodeWarrior warnings disabling

#pragma MESSAGE DISABLE C1420 /* C1420: Result of function-call is ignored */
#pragma MESSAGE DISABLE C5703 /* C5703: Parameter declared in function but not referenced */

//----------- Tasks ----------------------------------------------------------

#define  TASK_PROCESSING_PRIORITY    2
#define  TASK_SPI_PRIORITY           3
#define  TASK_IO_PRIORITY            4

#define  TASK_PROCESSING_STK_SIZE   64
#define  TASK_SPI_STK_SIZE          64
#define  TASK_IO_STK_SIZE           64

unsigned int task_processing_stack[TASK_PROCESSING_STK_SIZE];
unsigned int task_spi_stack[TASK_SPI_STK_SIZE];
unsigned int task_io_stack[TASK_IO_STK_SIZE];

TN_TCB  task_processing;
TN_TCB  task_spi;
TN_TCB  task_io;

void task_processing_func(void * par);
void task_spi_func(void * par);
void task_io_func(void * par);

//-------- OS - Semaphores ----------------------------------------------

TN_SEM semKBI;

//-------- OS - Queues --------------------------------------------------

#define  QUEUE_EVENTS_SIZE     8

   //--- UART RX queue

TN_DQUE  queueEvents;
void     * queueEventsMem[QUEUE_EVENTS_SIZE];



//--- For sharing int vectors with bootloader

#define NUM_INT_VECTORS  25
volatile unsigned char rint_vectors[3*NUM_INT_VECTORS] @0x100;

//-- External assembly function prototypes

void Vtpm1ovf_isr(void);
void Vrti_isr(void);
void Vkeyboard1_isr(void);

//------ Non-OS globals

tBTNSTATE    button1Status;
tBTNSTATE    button2Status;
tBTNSTATE    emergStatus;
tMOTIONSTATE motionStatus;

unsigned char gKbdIntReg;


//----------------------------------------------------------------------------
void main(void)
{
   UINT8 i,j;
   unsigned int * jmp_addr;

   gKbdIntReg = 0;

   HardwareInit();

/* Non- OS hardware test

   for(;;)
   {
      PTCD &= ~LED1_MASK;
      for(crc_val=0; crc_val < 100000;crc_val++);
      PTCD |= LED1_MASK;
      for(crc_val=0; crc_val < 100000;crc_val++);

   }
*/

   //--- Set int vectors to JMP RES

   j=0;
   for(i=0;i<NUM_INT_VECTORS;i++)
   {
      rint_vectors[j] = 0xCC;  //-- JMP opr16a
      j++;
      jmp_addr = (unsigned int *)&rint_vectors[j];
      *jmp_addr = 0xFFFF;
      j++;
      j++;
   }

   //-- Set active int vectors

   jmp_addr  = (unsigned int *)&rint_vectors[INT_Vtpm1ovf * 3 + 1];
   *jmp_addr = (unsigned int)Vtpm1ovf_isr;

   jmp_addr  = (unsigned int *)&rint_vectors[INT_Vswi * 3 + 1];
   *jmp_addr = (unsigned int)tn_switch_context_ex;

   jmp_addr  = (unsigned int *)&rint_vectors[INT_Vrti * 3 + 1];
   *jmp_addr = (unsigned int)Vrti_isr;

   jmp_addr  = (unsigned int *)&rint_vectors[INT_Vkeyboard * 3 + 1];
   *jmp_addr = (unsigned int)Vkeyboard1_isr;


   tn_start_system(); //-- Never returns
}

//----------------------------------------------------------------------------
void  tn_app_init()
{
   //--- Task processing

   tn_task_create(&task_processing,              //-- task TCB
                 task_processing_func,           //-- task function
                 TASK_PROCESSING_PRIORITY,       //-- task priority
                 &(task_processing_stack         //-- task stack first addr in memory
                    [TASK_PROCESSING_STK_SIZE-1]),
                 TASK_PROCESSING_STK_SIZE,       //-- task stack size (in int,not bytes)
                 NULL,                           //-- task function parameter
                 TN_TASK_START_ON_CREATION       //-- Creation option
                 );

   //--- Task SPI

   tn_task_create(&task_spi,               //-- task TCB
                 task_spi_func,            //-- task function
                 TASK_SPI_PRIORITY,        //-- task priority
                 &(task_spi_stack          //-- task stack first addr in memory
                    [TASK_SPI_STK_SIZE-1]),
                 TASK_SPI_STK_SIZE,        //-- task stack size (in int,not bytes)
                 NULL,                        //-- task function parameter
                 TN_TASK_START_ON_CREATION    //-- Creation option
                 );

   //--- Task IO

   tn_task_create(&task_io,                    //-- task TCB
                 task_io_func,                   //-- task function
                 TASK_IO_PRIORITY,               //-- task priority
                 &(task_io_stack                 //-- task stack first addr in memory
                    [TASK_IO_STK_SIZE-1]),
                 TASK_IO_STK_SIZE,               //-- task stack size (in int,not bytes)
                 NULL,                           //-- task function parameter
                 TN_TASK_START_ON_CREATION     //-- Creation option
                 );

  //--- Semaphores

   tn_sem_create(&semKBI,
                       0,   //-- Start value
                       1);  //-- Max value


  //--- Queues

   tn_queue_create(&queueEvents,           //-- Ptr to already existing TN_DQUE
                      &queueEventsMem[0],  //-- Ptr to already existing array of void * to store data queue entries.Can be NULL
                      QUEUE_EVENTS_SIZE    //-- Capacity of data queue(num entries).Can be 0
                    );

}

//----------------------------------------------------------------------------
void task_processing_func(void * par)
{
   INT8 rc;
   UINT8 i;

   unsigned int param;

   for(;;)
   {
      rc = tn_queue_receive(&queueEvents,(void **)&param,TN_WAIT_INFINITE);
      if(rc == TERR_NO_ERR)
      {
         rc = param >>8;
         if(rc == MASK_BUTTON1)
         {

            if((param & 0x00FF) == BTN_BELOW_2S)
               i = 16;
            else if((param & 0x00FF) == BTN_ABOVE_2S)
               i = 128;
            else
               i = 1;

            PTBD |= LED1_MASK;     //-- LED on
            tn_task_sleep(i);
            PTBD &= ~LED1_MASK;
         }
         else if(rc == MASK_BUTTON2)
         {
            if((param & 0x00FF) == BTN_BELOW_2S)
               rc = 1;
            else if((param & 0x00FF) == BTN_ABOVE_2S)
               rc = 4;
            else
               rc = 10;
         }
      }
   }
}

//----------------------------------------------------------------------------
void task_spi_func(void * par)
{
   for(;;)
   {
      tn_task_sleep(128);
   }
}

//----------------------------------------------------------------------------
void task_io_func(void * par)
{
   //INT8 rc;

   UINT32 timeout;
   timeout = 4; //TN_WAIT_INFINITE;
   for(;;)
   {

      //  Here we check buttons/motion or immediately
      //  after pressing(sem) or each XX milliseconds(timeout)


      tn_sem_acquire(&semKBI, timeout);


      processingButtons(&button1Status);
      processingButtons(&button2Status);
      processingMotion(&motionStatus);

      if(motionStatus.status == MTN_NO_MOTION)
          PTBD &= ~LED2_MASK;    //-- LED off
      else if(motionStatus.status == MTN_MOTION)
          PTBD |= LED2_MASK;     //-- LED on


   }
}
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
